@@ -41,7 +41,6 @@ from bson.dbref import DBRef
 from bson.py3compat import abc, iteritems, PY3, StringIO, text_type
 from bson.son import SON
 from bson.timestamp import Timestamp
-from bson.tz_util import FixedOffset
 from bson.errors import (InvalidBSON,
                          InvalidDocument,
                          InvalidStringData)
@@ -51,6 +50,7 @@ from bson.tz_util import (FixedOffset,
                           utc)
 
 from test import qcheck, SkipTest, unittest
+from test.utils import ExceptionCatchingThread
 
 if PY3:
     long = int
@@ -586,12 +586,12 @@ class TestBSON(unittest.TestCase):
 
     def test_small_long_encode_decode(self):
         encoded1 = BSON.encode({'x': 256})
-        decoded1 = BSON.decode(encoded1)['x']
+        decoded1 = BSON(encoded1).decode()['x']
         self.assertEqual(256, decoded1)
         self.assertEqual(type(256), type(decoded1))
 
         encoded2 = BSON.encode({'x': Int64(256)})
-        decoded2 = BSON.decode(encoded2)['x']
+        decoded2 = BSON(encoded2).decode()['x']
         expected = Int64(256)
         self.assertEqual(expected, decoded2)
         self.assertEqual(type(expected), type(decoded2))
@@ -905,6 +905,38 @@ class TestBSON(unittest.TestCase):
                           {"_id": {'$oid': "52d0b971b3ba219fdeb4170e"}}, True)
         BSON.encode({"_id": {'$oid': "52d0b971b3ba219fdeb4170e"}})
 
+    def test_bson_encode_thread_safe(self):
+
+        def target(i):
+            for j in range(1000):
+                my_int = type('MyInt_%s_%s' % (i, j), (int,), {})
+                bson.BSON.encode({'my_int': my_int()})
+
+        threads = [ExceptionCatchingThread(target=target, args=(i,))
+                   for i in range(3)]
+        for t in threads:
+            t.start()
+
+        for t in threads:
+            t.join()
+
+        for t in threads:
+            self.assertIsNone(t.exc)
+
+    def test_raise_invalid_document(self):
+        class Wrapper(object):
+            def __init__(self, val):
+                self.val = val
+
+            def __repr__(self):
+                return repr(self.val)
+
+        self.assertEqual('1', repr(Wrapper(1)))
+        with self.assertRaisesRegex(
+                InvalidDocument,
+                "cannot encode object: 1, of type: " + repr(Wrapper)):
+            BSON.encode({'t': Wrapper(1)})
+
 
 class TestCodecOptions(unittest.TestCase):
     def test_document_class(self):
@@ -931,7 +963,8 @@ class TestCodecOptions(unittest.TestCase):
         r = ("CodecOptions(document_class=dict, tz_aware=False, "
              "uuid_representation=PYTHON_LEGACY, "
              "unicode_decode_error_handler='strict', "
-             "tzinfo=None)")
+             "tzinfo=None, type_registry=TypeRegistry(type_codecs=[], "
+             "fallback_encoder=None))")
         self.assertEqual(r, repr(CodecOptions()))
 
     def test_decode_all_defaults(self):
@@ -956,63 +989,63 @@ class TestCodecOptions(unittest.TestCase):
         replaced_key = b'ke\xe9str'.decode('utf-8', 'replace')
         ignored_key = b'ke\xe9str'.decode('utf-8', 'ignore')
 
-        dec = BSON.decode(invalid_key, CodecOptions(
+        dec = BSON(invalid_key).decode(CodecOptions(
             unicode_decode_error_handler="replace"))
         self.assertEqual(dec, {replaced_key: u"foobar"})
 
-        dec = BSON.decode(invalid_key, CodecOptions(
+        dec = BSON(invalid_key).decode(CodecOptions(
             unicode_decode_error_handler="ignore"))
         self.assertEqual(dec, {ignored_key: u"foobar"})
 
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_key, CodecOptions(
+        self.assertRaises(InvalidBSON, BSON(invalid_key).decode, CodecOptions(
             unicode_decode_error_handler="strict"))
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_key,
+        self.assertRaises(InvalidBSON, BSON(invalid_key).decode,
                           CodecOptions())
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_key)
+        self.assertRaises(InvalidBSON, BSON(invalid_key).decode)
 
         # Test handing of bad string value.
         invalid_val = BSON(enc[:18] + b'\xe9' + enc[19:])
         replaced_val = b'fo\xe9bar'.decode('utf-8', 'replace')
         ignored_val = b'fo\xe9bar'.decode('utf-8', 'ignore')
 
-        dec = BSON.decode(invalid_val, CodecOptions(
+        dec = BSON(invalid_val).decode(CodecOptions(
             unicode_decode_error_handler="replace"))
         self.assertEqual(dec, {u"keystr": replaced_val})
 
-        dec = BSON.decode(invalid_val, CodecOptions(
+        dec = BSON(invalid_val).decode(CodecOptions(
             unicode_decode_error_handler="ignore"))
         self.assertEqual(dec, {u"keystr": ignored_val})
 
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_val, CodecOptions(
+        self.assertRaises(InvalidBSON, BSON(invalid_val).decode, CodecOptions(
             unicode_decode_error_handler="strict"))
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_val,
+        self.assertRaises(InvalidBSON, BSON(invalid_val).decode,
                           CodecOptions())
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_val)
+        self.assertRaises(InvalidBSON, BSON(invalid_val).decode)
 
         # Test handing bad key + bad value.
         invalid_both = BSON(
             enc[:7] + b'\xe9' + enc[8:18] + b'\xe9' + enc[19:])
 
-        dec = BSON.decode(invalid_both, CodecOptions(
+        dec = BSON(invalid_both).decode(CodecOptions(
             unicode_decode_error_handler="replace"))
         self.assertEqual(dec, {replaced_key: replaced_val})
 
-        dec = BSON.decode(invalid_both, CodecOptions(
+        dec = BSON(invalid_both).decode(CodecOptions(
             unicode_decode_error_handler="ignore"))
         self.assertEqual(dec, {ignored_key: ignored_val})
 
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_both, CodecOptions(
+        self.assertRaises(InvalidBSON, BSON(invalid_both).decode, CodecOptions(
             unicode_decode_error_handler="strict"))
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_both,
+        self.assertRaises(InvalidBSON, BSON(invalid_both).decode,
                           CodecOptions())
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_both)
+        self.assertRaises(InvalidBSON, BSON(invalid_both).decode)
 
         # Test handling bad error mode.
-        dec = BSON.decode(enc, CodecOptions(
+        dec = BSON(enc).decode(CodecOptions(
             unicode_decode_error_handler="junk"))
         self.assertEqual(dec, {"keystr": "foobar"})
 
-        self.assertRaises(InvalidBSON, BSON.decode, invalid_both,
+        self.assertRaises(InvalidBSON, BSON(invalid_both).decode,
                           CodecOptions(unicode_decode_error_handler="junk"))
 
 
