@@ -132,6 +132,12 @@ class TestCollection(IntegrationTest):
     def tearDownClass(cls):
         cls.db.drop_collection("test_large_limit")
 
+    def setUp(self):
+        self.db.test.drop()
+
+    def tearDown(self):
+        self.db.test.drop()
+
     @contextlib.contextmanager
     def write_concern_collection(self):
         if client_context.version.at_least(3, 3, 9) and client_context.is_rs:
@@ -1316,6 +1322,21 @@ class TestCollection(IntegrationTest):
                           db.test.insert_many,
                           [{"text": text}])
 
+    def test_write_error_unicode(self):
+        coll = self.db.test
+        self.addCleanup(coll.drop)
+
+        coll.create_index('a', unique=True)
+        coll.insert_one({'a': u'unicode \U0001f40d'})
+        with self.assertRaisesRegex(
+                DuplicateKeyError,
+                'E11000 duplicate key error') as ctx:
+            coll.insert_one({'a': u'unicode \U0001f40d'})
+
+        # Once more for good measure.
+        self.assertIn('E11000 duplicate key error',
+                      str(ctx.exception))
+
     def test_wtimeout(self):
         # Ensure setting wtimeout doesn't disable write concern altogether.
         # See SERVER-12596.
@@ -1616,9 +1637,8 @@ class TestCollection(IntegrationTest):
         self.assertEqual([{'foo': [1, 2]}], list(result))
 
         # Test write concern.
-        out_pipeline = [pipeline, {'$out': 'output-collection'}]
         with self.write_concern_collection() as coll:
-            coll.aggregate(out_pipeline)
+            coll.aggregate([{'$out': 'output-collection'}])
 
     def test_aggregate_raw_bson(self):
         db = self.db
@@ -1751,9 +1771,8 @@ class TestCollection(IntegrationTest):
         db.test_large_limit.create_index([('x', 1)])
         my_str = "mongomongo" * 1000
 
-        for i in range(2000):
-            doc = {"x": i, "y": my_str}
-            db.test_large_limit.insert_one(doc)
+        db.test_large_limit.insert_many(
+            {"x": i, "y": my_str} for i in range(2000))
 
         i = 0
         y = 0
@@ -1767,9 +1786,7 @@ class TestCollection(IntegrationTest):
     def test_find_kwargs(self):
         db = self.db
         db.drop_collection("test")
-
-        for i in range(10):
-            db.test.insert_one({"x": i})
+        db.test.insert_many({"x": i} for i in range(10))
 
         self.assertEqual(10, db.test.count_documents({}))
 
@@ -1794,8 +1811,7 @@ class TestCollection(IntegrationTest):
         self.assertEqual(0, db.test.count_documents({}))
         self.assertEqual(0, db.foo.count_documents({}))
 
-        for i in range(10):
-            db.test.insert_one({"x": i})
+        db.test.insert_many({"x": i} for i in range(10))
 
         self.assertEqual(10, db.test.count_documents({}))
 
@@ -1990,10 +2006,13 @@ class TestCollection(IntegrationTest):
         self.db.test.insert_many([{"x": 1}, {"x": 2}])
         self.db.test.create_index("x")
 
-        self.assertEqual(1, len(list(self.db.test.find({"$min": {"x": 2},
-                                                        "$query": {}}))))
-        self.assertEqual(2, self.db.test.find({"$min": {"x": 2},
-                                               "$query": {}})[0]["x"])
+        cursor = self.db.test.find({"$min": {"x": 2}, "$query": {}})
+        if client_context.requires_hint_with_min_max_queries:
+            cursor = cursor.hint("x_1")
+
+        docs = list(cursor)
+        self.assertEqual(1, len(docs))
+        self.assertEqual(2, docs[0]["x"])
 
     def test_numerous_inserts(self):
         # Ensure we don't exceed server's 1000-document batch size limit.
