@@ -21,6 +21,7 @@ import re
 import sys
 import time
 import threading
+import warnings
 
 sys.path[0:0] = [""]
 
@@ -357,7 +358,7 @@ class TestCursor(IntegrationTest):
         self.assertTrue(coll.find().explain())
         started = listener.results['started']
         self.assertEqual(len(started), 1)
-        self.assertNotIn("readConern", started[0].command)
+        self.assertNotIn("readConcern", started[0].command)
 
     def test_hint(self):
         db = self.db
@@ -449,66 +450,102 @@ class TestCursor(IntegrationTest):
             break
         self.assertRaises(InvalidOperation, a.limit, 5)
 
+    @ignore_deprecations  # Ignore max without hint.
     def test_max(self):
         db = self.db
         db.test.drop()
-        db.test.create_index([("j", ASCENDING)])
+        j_index = [("j", ASCENDING)]
+        db.test.create_index(j_index)
 
         db.test.insert_many([{"j": j, "k": j} for j in range(10)])
 
-        cursor = db.test.find().max([("j", 3)])
+        def find(max_spec, expected_index):
+            cursor = db.test.find().max(max_spec)
+            if client_context.requires_hint_with_min_max_queries:
+                cursor = cursor.hint(expected_index)
+            return cursor
+
+        cursor = find([("j", 3)], j_index)
         self.assertEqual(len(list(cursor)), 3)
 
         # Tuple.
-        cursor = db.test.find().max((("j", 3), ))
+        cursor = find((("j", 3),), j_index)
         self.assertEqual(len(list(cursor)), 3)
 
         # Compound index.
-        db.test.create_index([("j", ASCENDING), ("k", ASCENDING)])
-        cursor = db.test.find().max([("j", 3), ("k", 3)])
+        index_keys = [("j", ASCENDING), ("k", ASCENDING)]
+        db.test.create_index(index_keys)
+        cursor = find([("j", 3), ("k", 3)], index_keys)
         self.assertEqual(len(list(cursor)), 3)
 
         # Wrong order.
-        cursor = db.test.find().max([("k", 3), ("j", 3)])
+        cursor = find([("k", 3), ("j", 3)], index_keys)
         self.assertRaises(OperationFailure, list, cursor)
 
         # No such index.
-        cursor = db.test.find().max([("k", 3)])
+        cursor = find([("k", 3)], "k")
         self.assertRaises(OperationFailure, list, cursor)
 
         self.assertRaises(TypeError, db.test.find().max, 10)
         self.assertRaises(TypeError, db.test.find().max, {"j": 10})
 
+    @ignore_deprecations  # Ignore min without hint.
     def test_min(self):
         db = self.db
         db.test.drop()
-        db.test.create_index([("j", ASCENDING)])
+        j_index = [("j", ASCENDING)]
+        db.test.create_index(j_index)
 
         db.test.insert_many([{"j": j, "k": j} for j in range(10)])
 
-        cursor = db.test.find().min([("j", 3)])
+        def find(min_spec, expected_index):
+            cursor = db.test.find().min(min_spec)
+            if client_context.requires_hint_with_min_max_queries:
+                cursor = cursor.hint(expected_index)
+            return cursor
+
+        cursor = find([("j", 3)], j_index)
         self.assertEqual(len(list(cursor)), 7)
 
         # Tuple.
-        cursor = db.test.find().min((("j", 3), ))
+        cursor = find((("j", 3),), j_index)
         self.assertEqual(len(list(cursor)), 7)
 
         # Compound index.
-        db.test.create_index([("j", ASCENDING), ("k", ASCENDING)])
-        cursor = db.test.find().min([("j", 3), ("k", 3)])
+        index_keys = [("j", ASCENDING), ("k", ASCENDING)]
+        db.test.create_index(index_keys)
+        cursor = find([("j", 3), ("k", 3)], index_keys)
         self.assertEqual(len(list(cursor)), 7)
 
         # Wrong order.
-        cursor = db.test.find().min([("k", 3), ("j", 3)])
+        cursor = find([("k", 3), ("j", 3)], index_keys)
         self.assertRaises(OperationFailure, list, cursor)
 
         # No such index.
-        cursor = db.test.find().min([("k", 3)])
+        cursor = find([("k", 3)], "k")
         self.assertRaises(OperationFailure, list, cursor)
 
         self.assertRaises(TypeError, db.test.find().min, 10)
         self.assertRaises(TypeError, db.test.find().min, {"j": 10})
 
+    @client_context.require_version_max(4, 1, -1)
+    def test_min_max_without_hint(self):
+        coll = self.db.test
+        j_index = [("j", ASCENDING)]
+        coll.create_index(j_index)
+
+        with warnings.catch_warnings(record=True) as warns:
+            warnings.simplefilter("default", DeprecationWarning)
+            list(coll.find().min([("j", 3)]))
+            self.assertIn('using a min/max query operator', str(warns[0]))
+            # Ensure the warning is raised with the proper stack level.
+            del warns[:]
+            list(coll.find().min([("j", 3)]))
+            self.assertIn('using a min/max query operator', str(warns[0]))
+            del warns[:]
+            list(coll.find().max([("j", 3)]))
+            self.assertIn('using a min/max query operator', str(warns[0]))
+
     def test_batch_size(self):
         db = self.db
         db.test.drop()
@@ -1220,9 +1257,6 @@ class TestCursor(IntegrationTest):
     @client_context.require_no_mongos
     @ignore_deprecations
     def test_comment(self):
-        if client_context.auth_enabled:
-            raise SkipTest("SERVER-4754 - This test uses profiling.")
-
         # MongoDB 3.1.5 changed the ns for commands.
         regex = {'$regex': r'pymongo_test.(\$cmd|test)'}
 
@@ -1246,14 +1280,14 @@ class TestCursor(IntegrationTest):
             op = self.db.system.profile.find({'ns': regex,
                                               'op': 'command',
                                               'command.count': 'test',
-                                              'command.$comment': 'foo'})
+                                              'command.comment': 'foo'})
             self.assertEqual(op.count(), 1)
 
             self.db.test.find().comment('foo').distinct('type')
             op = self.db.system.profile.find({'ns': regex,
                                               'op': 'command',
                                               'command.distinct': 'test',
-                                              'command.$comment': 'foo'})
+                                              'command.comment': 'foo'})
             self.assertEqual(op.count(), 1)
         finally:
             self.db.set_profiling_level(OFF)
