@@ -56,7 +56,9 @@ from test.utils import (ignore_deprecations,
                         rs_or_single_client_noauth,
                         rs_or_single_client,
                         server_started_with_auth,
-                        IMPOSSIBLE_WRITE_CONCERN)
+                        IMPOSSIBLE_WRITE_CONCERN,
+                        OvertCommandListener)
+from test.test_custom_types import DECIMAL_CODECOPTS
 
 
 if PY3:
@@ -156,7 +158,7 @@ class TestDatabase(IntegrationTest):
         self.assertTrue(u"test.foo" in db.list_collection_names())
         self.assertRaises(CollectionInvalid, db.create_collection, "test.foo")
 
-    def _test_collection_names(self, meth, test_no_system):
+    def _test_collection_names(self, meth, **no_system_kwargs):
         db = Database(self.client, "pymongo_test")
         db.test.insert_one({"dummy": u"object"})
         db.test.mike.insert_one({"dummy": u"object"})
@@ -167,13 +169,11 @@ class TestDatabase(IntegrationTest):
         for coll in colls:
             self.assertTrue("$" not in coll)
 
-        if test_no_system:
-            db.systemcoll.test.insert_one({})
-            no_system_collections = getattr(
-                db, meth)(include_system_collections=False)
-            for coll in no_system_collections:
-                self.assertTrue(not coll.startswith("system."))
-            self.assertIn("systemcoll.test", no_system_collections)
+        db.systemcoll.test.insert_one({})
+        no_system_collections = getattr(db, meth)(**no_system_kwargs)
+        for coll in no_system_collections:
+            self.assertTrue(not coll.startswith("system."))
+        self.assertIn("systemcoll.test", no_system_collections)
 
         # Force more than one batch.
         db = self.client.many_collections
@@ -186,10 +186,45 @@ class TestDatabase(IntegrationTest):
             self.client.drop_database("many_collections")
 
     def test_collection_names(self):
-        self._test_collection_names('collection_names', True)
+        self._test_collection_names(
+            'collection_names', include_system_collections=False)
 
     def test_list_collection_names(self):
-        self._test_collection_names('list_collection_names', False)
+        self._test_collection_names(
+            'list_collection_names', filter={
+                "name": {"$regex": r"^(?!system\.)"}})
+
+    def test_list_collection_names_filter(self):
+        listener = OvertCommandListener()
+        results = listener.results
+        client = rs_or_single_client(event_listeners=[listener])
+        db = client[self.db.name]
+        db.capped.drop()
+        db.create_collection("capped", capped=True, size=4096)
+        db.capped.insert_one({})
+        db.non_capped.insert_one({})
+        self.addCleanup(client.drop_database, db.name)
+
+        # Should not send nameOnly.
+        for filter in ({'options.capped': True},
+                       {'options.capped': True, 'name': 'capped'}):
+            results.clear()
+            names = db.list_collection_names(filter=filter)
+            self.assertEqual(names, ["capped"])
+            self.assertNotIn("nameOnly", results["started"][0].command)
+
+        # Should send nameOnly (except on 2.6).
+        for filter in (None, {}, {'name': {'$in': ['capped', 'non_capped']}}):
+            results.clear()
+            names = db.list_collection_names(filter=filter)
+            self.assertIn("capped", names)
+            self.assertIn("non_capped", names)
+            command = results["started"][0].command
+            if client_context.version >= (3, 0):
+                self.assertIn("nameOnly", command)
+                self.assertTrue(command["nameOnly"])
+            else:
+                self.assertNotIn("nameOnly", command)
 
     def test_list_collections(self):
         self.client.drop_database("pymongo_test")
@@ -391,23 +426,27 @@ class TestDatabase(IntegrationTest):
         self.assertTrue(isinstance(info[0]["ts"], datetime.datetime))
 
     @client_context.require_no_mongos
+    @ignore_deprecations
     def test_errors(self):
-        with ignore_deprecations():
-            # We must call getlasterror, etc. on same socket as last operation.
-            db = rs_or_single_client(maxPoolSize=1).pymongo_test
-            db.reset_error_history()
-            self.assertEqual(None, db.error())
+        # We must call getlasterror, etc. on same socket as last operation.
+        db = rs_or_single_client(maxPoolSize=1).pymongo_test
+        db.reset_error_history()
+        self.assertEqual(None, db.error())
+        if client_context.supports_getpreverror:
             self.assertEqual(None, db.previous_error())
 
-            db.test.insert_one({"_id": 1})
-            unacked = db.test.with_options(write_concern=WriteConcern(w=0))
+        db.test.insert_one({"_id": 1})
+        unacked = db.test.with_options(write_concern=WriteConcern(w=0))
 
-            unacked.insert_one({"_id": 1})
-            self.assertTrue(db.error())
+        unacked.insert_one({"_id": 1})
+        self.assertTrue(db.error())
+        if client_context.supports_getpreverror:
             self.assertTrue(db.previous_error())
 
-            unacked.insert_one({"_id": 1})
-            self.assertTrue(db.error())
+        unacked.insert_one({"_id": 1})
+        self.assertTrue(db.error())
+
+        if client_context.supports_getpreverror:
             prev_error = db.previous_error()
             self.assertEqual(prev_error["nPrev"], 1)
             del prev_error["nPrev"]
@@ -419,13 +458,15 @@ class TestDatabase(IntegrationTest):
             error.pop("connectionId", None)
             self.assertEqualReply(error, prev_error)
 
-            db.test.find_one()
-            self.assertEqual(None, db.error())
+        db.test.find_one()
+        self.assertEqual(None, db.error())
+        if client_context.supports_getpreverror:
             self.assertTrue(db.previous_error())
             self.assertEqual(db.previous_error()["nPrev"], 2)
 
-            db.reset_error_history()
-            self.assertEqual(None, db.error())
+        db.reset_error_history()
+        self.assertEqual(None, db.error())
+        if client_context.supports_getpreverror:
             self.assertEqual(None, db.previous_error())
 
     def test_command(self):
@@ -434,13 +475,8 @@ class TestDatabase(IntegrationTest):
         first = db.command("buildinfo")
         second = db.command({"buildinfo": 1})
         third = db.command("buildinfo", 1)
-        # The logicalTime and operationTime fields were introduced in MongoDB
-        # 3.5. Their value can change from one command call to the next.
-        for doc in (first, second, third):
-            doc.pop("logicalTime", None)
-            doc.pop("operationTime", None)
-        self.assertEqual(first, second)
-        self.assertEqual(second, third)
+        self.assertEqualReply(first, second)
+        self.assertEqualReply(second, third)
 
     # We use 'aggregate' as our example command, since it's an easy way to
     # retrieve a BSON regex from a collection using a command. But until
@@ -941,6 +977,45 @@ class TestDatabase(IntegrationTest):
                                       "maxTimeAlwaysTimeOut",
                                       mode="off")
 
+    def test_with_options(self):
+        codec_options = DECIMAL_CODECOPTS
+        read_preference = ReadPreference.SECONDARY_PREFERRED
+        write_concern = WriteConcern(j=True)
+        read_concern = ReadConcern(level="majority")
+
+        # List of all options to compare.
+        allopts = ['name', 'client', 'codec_options',
+                   'read_preference', 'write_concern', 'read_concern']
+
+        db1 = self.client.get_database(
+            'with_options_test', codec_options=codec_options,
+            read_preference=read_preference, write_concern=write_concern,
+            read_concern=read_concern)
+
+        # Case 1: swap no options
+        db2 = db1.with_options()
+        for opt in allopts:
+            self.assertEqual(getattr(db1, opt), getattr(db2, opt))
+
+        # Case 2: swap all options
+        newopts = {'codec_options': CodecOptions(),
+                   'read_preference': ReadPreference.PRIMARY,
+                   'write_concern': WriteConcern(w=1),
+                   'read_concern': ReadConcern(level="local")}
+        db2 = db1.with_options(**newopts)
+        for opt in newopts:
+            self.assertEqual(
+                getattr(db2, opt), newopts.get(opt, getattr(db1, opt)))
+
+    def test_current_op_codec_options(self):
+        class MySON(SON):
+            pass
+        opts = CodecOptions(document_class=MySON)
+        db = self.client.get_database("pymongo_test", codec_options=opts)
+        current_op = db.current_op(True)
+        self.assertTrue(current_op['inprog'])
+        self.assertIsInstance(current_op, MySON)
+
 
 if __name__ == "__main__":
     unittest.main()
