@@ -139,6 +139,16 @@ def get_topology_type_name(scenario_def):
         return name
 
 
+def get_topology_settings_dict(**kwargs):
+    settings = dict(
+        monitor_class=MockMonitor,
+        heartbeat_frequency=HEARTBEAT_FREQUENCY,
+        pool_class=MockPool
+    )
+    settings.update(kwargs)
+    return settings
+
+
 def create_test(scenario_def):
     def run_scenario(self):
         # Initialize topologies.
@@ -147,14 +157,14 @@ def create_test(scenario_def):
         else:
             frequency = HEARTBEAT_FREQUENCY
 
-        settings = dict(
-            monitor_class=MockMonitor,
-            heartbeat_frequency=frequency,
-            pool_class=MockPool)
-
-        settings['seeds'], hosts = get_addresses(
+        seeds, hosts = get_addresses(
             scenario_def['topology_description']['servers'])
 
+        settings = get_topology_settings_dict(
+            heartbeat_frequency=frequency,
+            seeds=seeds
+        )
+
         # "Eligible servers" is defined in the server selection spec as
         # the set of servers matching both the ReadPreference's mode
         # and tag sets.
@@ -180,23 +190,14 @@ def create_test(scenario_def):
         else:
             # Make first letter lowercase to match read_pref's modes.
             pref_def = scenario_def['read_preference']
-            mode_string = pref_def.get('mode', 'primary')
-            mode_string = mode_string[:1].lower() + mode_string[1:]
-            mode = read_preferences.read_pref_mode_from_name(mode_string)
-            max_staleness = pref_def.get('maxStalenessSeconds', -1)
-            tag_sets = pref_def.get('tag_sets')
-
             if scenario_def.get('error'):
                 with self.assertRaises((ConfigurationError, ValueError)):
                     # Error can be raised when making Read Pref or selecting.
-                    pref = read_preferences.make_read_preference(
-                        mode, tag_sets=tag_sets, max_staleness=max_staleness)
-
+                    pref = parse_read_preference(pref_def)
                     top_latency.select_server(pref)
                 return
 
-            pref = read_preferences.make_read_preference(
-                mode, tag_sets=tag_sets, max_staleness=max_staleness)
+            pref = parse_read_preference(pref_def)
 
         # Select servers.
         if not scenario_def.get('suitable_servers'):
@@ -281,3 +282,14 @@ def create_selection_tests(test_dir):
             setattr(TestAllScenarios, new_test.__name__, new_test)
 
     return TestAllScenarios
+
+
+def parse_read_preference(pref):
+    # Make first letter lowercase to match read_pref's modes.
+    mode_string = pref.get('mode', 'primary')
+    mode_string = mode_string[:1].lower() + mode_string[1:]
+    mode = read_preferences.read_pref_mode_from_name(mode_string)
+    max_staleness = pref.get('maxStalenessSeconds', -1)
+    tag_sets = pref.get('tag_sets')
+    return read_preferences.make_read_preference(
+        mode, tag_sets=tag_sets, max_staleness=max_staleness)
